'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.default = combineReducers;

var _lodash = require('lodash.reduce');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.foreach');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.mapvalues');

var _lodash6 = _interopRequireDefault(_lodash5);

var _lodash7 = require('lodash.size');

var _lodash8 = _interopRequireDefault(_lodash7);

var _immutable = require('immutable');

var _mindfrontReduxUtils = require('mindfront-redux-utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function combineReducers(reducers) {
  var createInitialState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _immutable.Map;

  var actionHandlers = {};
  var otherReducers = {};
  var initialStateObj = {};

  // regroup reducers that have actionHandlers:
  //   from: prop name -> action type -> reducer
  //     to: action type -> prop name -> reducer
  // put reducers without actionHandlers into a prop name -> reducer map

  (0, _lodash4.default)(reducers, function (reducer, key) {
    initialStateObj[key] = reducer(undefined, {});
    if (reducer.actionHandlers) {
      (0, _lodash4.default)(reducer.actionHandlers, function (actionHandler, type) {
        (actionHandlers[type] || (actionHandlers[type] = {}))[key] = actionHandler;
      });
    } else otherReducers[key] = reducer;
  });
  var initialState = createInitialState(initialStateObj);

  // now we can create a more efficient reducer for each action type
  // and one more efficient reducer for the otherReducers

  // creates an efficient reducer from a prop name -> reducer map
  var combineBase = function combineBase(reducers) {
    var result = void 0;
    if ((0, _lodash8.default)(reducers) > 1) {
      // we have to update multiple keys; use state.withMutations
      result = function result() {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments[1];
        return state.withMutations(function (mutableState) {
          return (0, _lodash2.default)(reducers, function (nextState, reducer, key) {
            return nextState.update(key, function (value) {
              return reducer(value, action);
            });
          }, mutableState);
        });
      };
    } else {
      // we only have to update one key; use state.update
      var key = (0, _keys2.default)(reducers)[0];
      var reducer = reducers[key];
      result = function result() {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments[1];
        return state.update(key, function (value) {
          return reducer(value, action);
        });
      };
    }
    result.domainHandlers = reducers;
    return result;
  };

  var actionHandlerReducer = (0, _lodash8.default)(actionHandlers) ? (0, _mindfrontReduxUtils.createReducer)(initialState, (0, _lodash6.default)(actionHandlers, combineBase)) : undefined;

  var otherReducer = (0, _lodash8.default)(otherReducers) ? combineBase(otherReducers) : undefined;

  if (actionHandlerReducer && otherReducer) return (0, _mindfrontReduxUtils.composeReducers)(actionHandlerReducer, otherReducer);
  if (actionHandlerReducer) return actionHandlerReducer;
  if (otherReducer) return otherReducer;
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    return state;
  };
}